"use strict";
var Signal_1 = require("./Signal");
/** Queue of functions to execute when idle (deferred functions) */
var deferred = [];
var deferredArgs = [];
/** True if currently handling deferred functions */
var runningDeferred = false;
/** True if already scheduled next handler */
var scheduled;
/** Current setTimeout ID, if >= 0 */
var deferTimeout = -1;
/** True if postMessage works, too */
var canPostMessage;
/** True if native Promise exists */
var canPromise = (typeof window !== "undefined") &&
    (typeof window.Promise !== "undefined");
/** Run a batch of deferred functions */
function _runDeferred(nonSched) {
    if (!nonSched)
        scheduled = false;
    if (deferTimeout >= 0) {
        clearTimeout(deferTimeout);
        deferTimeout = -1;
    }
    // run a number of deferred functions
    runningDeferred = true;
    var max = Math.max(100, deferred.length >> 2);
    for (var i = 0; i < deferred.length && i < max; i++) {
        try {
            deferred[i].apply(undefined, deferredArgs[i]);
        }
        catch (err) {
            Signal_1.UnhandledException(err);
        }
    }
    runningDeferred = false;
    // reschedule if necessary
    if (i < deferred.length) {
        deferred.splice(0, i);
        deferredArgs.splice(0, i);
        _reschedule();
    }
    else {
        deferred.length = 0;
        deferredArgs.length = 0;
    }
}
/** Schedule a deferred function run */
function _reschedule() {
    if (!scheduled && !runningDeferred) {
        scheduled = true;
        if (canPromise)
            window.Promise.resolve().then(_runDeferred);
        if (canPostMessage)
            window.postMessage("yield", "*");
        else
            deferTimeout = setTimeout(_runDeferred, 0);
    }
}
/** Execute given function only when idle, with given arguments (array or `arguments` object) if any */
function defer(f, args) {
    deferred.push(f);
    deferredArgs.push(args);
    _reschedule();
}
exports.defer = defer;
/** Run a batch of deferred functions; returns true if there are still more deferred functions in the queue (or false to stop infinite recursion if already running) */
function yieldAll() {
    if (runningDeferred)
        return false;
    _runDeferred(true);
    return (deferred.length > 0);
}
exports.yieldAll = yieldAll;
// check if postMessage works in this browser and set flag
if (typeof window === "object") {
    window.addEventListener("message", function () {
        canPostMessage = true;
        _runDeferred();
    });
    setTimeout(function () { return window.postMessage("yield", "*"); }, 1);
}
